@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2003-2005 by liuspider                                  *
+ *   Copyright (C) 2003-2006 by liuspider                                  *
  *   liuspider@users.sourceforge.net                                       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -36,6 +36,8 @@
 #include <kdeversion.h>
 #define KDEGT340 (KDE::version() >= KDE_MAKE_VERSION(3,4,0))
 
+#define ACTIONLIST_FRONTEND "Frontend Properties"
+#define ACTIONLIST_GUI "GUI Properties"
 #include <qxembed.h>
 
 MainWindow::MainWindow( QWidget* parent, const char* name, WFlags fl )
@@ -43,8 +45,8 @@ MainWindow::MainWindow( QWidget* parent, const char* name, WFlags fl )
     ScimXMLGUIClient(SkimPluginManager::self()->getActionCollection()), DCOPObject("Skim_MainWindow"),
     m_autoHideTimeoutTimer(0), m_showHandleTimer(0), m_showExtHandleTimer(0),m_mode(PanelEmbedded),
     m_embedInitialized(false), m_contentIsVisible( false ), m_panelTurnedOn( false ),
-    m_updatePropertiesNeeded(false), m_updateGUIProperties(false), m_updateFrontendProperties(false),
-    m_applet_direction(Qt::Horizontal), m_shouldChangeDirection(false), m_logo(0) {
+    m_updatePropertiesNeeded(false), m_applet_direction(Qt::Horizontal), 
+    m_shouldChangeDirection(false), m_logo(0) {
 
     QXEmbed::initialize ();
 
@@ -105,8 +107,8 @@ MainWindow::MainWindow( QWidget* parent, const char* name, WFlags fl )
     connect(m_defaultActionCollection, SIGNAL(propertiesRegistered(bool)), SLOT(updateProperties (bool)));
     connect(m_defaultActionCollection, SIGNAL(propertiesRemoved(bool)), SLOT(updateProperties (bool)));
     connect(m_defaultActionCollection, SIGNAL(propertyChanged(bool)), SLOT(updateProperties (bool)));
-    connect(m_inputServer, SIGNAL( turnOnPanelReq() ), this, SLOT( slotTurnOn() ));
-    connect(m_inputServer, SIGNAL( turnOffPanelReq() ), this, SLOT( slotTurnOff() ));
+    connect(m_inputServer, SIGNAL( turnOnPanelReq() ), SLOT( slotTurnOn() ));
+    connect(m_inputServer, SIGNAL( turnOffPanelReq() ), SLOT( slotTurnOff() ));
 
     #if ENABLE_DEBUG
     show();
@@ -115,8 +117,10 @@ MainWindow::MainWindow( QWidget* parent, const char* name, WFlags fl )
     connect(m_allModules->getActionCollection(), SIGNAL(standaloneHelperActionsChanged()),
         this, SLOT(initContextMenu()));
 
-    connect( m_inputServer, SIGNAL( transaction_start() ), this, SLOT( disableUpdates() ) );
-    connect( m_inputServer, SIGNAL( transaction_end() ), this, SLOT( enableUpdates() ) );
+    //fix bug: https://bugzilla.novell.com/show_bug.cgi?id=149541
+    //KXMLGUIClient keeps a list of pluged actions, if actions are deleted, this list
+    //won't be changed, so we have to unplug all actions before remove any actions 
+    connect( m_defaultActionCollection, SIGNAL( preparePropertiesRemove(bool) ), SLOT( emptyToolbar(bool) ) );
 
     initContextMenu();
 }
@@ -128,30 +132,15 @@ MainWindow::~MainWindow() {
         ScimKdeSettings::setMainWindow_Direction( (int)m_mainWindowLayout->direction() );
     }
 
-    ScimKdeSettings::writeConfig();
     m_contextMenu->clear();
 }
 
-void MainWindow::disableUpdates()
+void MainWindow::emptyToolbar(bool isFrontEndProperties)
 {
-    setUpdatesEnabled( false );
-    m_toolbar->setUpdatesEnabled( false );
-}
-
-void MainWindow::enableUpdates()
-{
-    setUpdatesEnabled( true );
-    m_toolbar->setUpdatesEnabled( true );
-
-    if(m_updateGUIProperties)
-        updateProperties(false);
-
-    if(m_updateFrontendProperties)
-        updateProperties(true);
-
-    //FIXME: see the FIXME in MainWindow::show()
-    if(isVisible())
-        QTimer::singleShot( 100, this, SLOT( adjustSize() ) );
+    if(isFrontEndProperties)
+        unplugActionList(ACTIONLIST_FRONTEND);
+    else
+        unplugActionList(ACTIONLIST_GUI);
 }
 
 void MainWindow::initContextMenu() {
@@ -189,43 +178,32 @@ void MainWindow::createToolbar () {
 }
 
 void MainWindow::updateProperties (bool isFrontEndProperties) {
-    if(isUpdatesEnabled())
-    {
-        if(isFrontEndProperties)
-            m_updateFrontendProperties = false;
-        else
-            m_updateGUIProperties = false;
+    KActionPtrList exta = isFrontEndProperties ?
+            m_defaultActionCollection->frontendPropertyActions() : m_defaultActionCollection->guiPropertyActions();
+    m_actionListName = isFrontEndProperties ? ACTIONLIST_FRONTEND : ACTIONLIST_GUI;
 
-        KActionPtrList exta = isFrontEndProperties ?
-                m_defaultActionCollection->frontendPropertyActions() : m_defaultActionCollection->guiPropertyActions();
-        m_actionListName = isFrontEndProperties ? "Frontend Properties" : "GUI Properties";
-
-        m_insertedActions.clear();
-        for(uint i=0; i<exta.size(); i++) {
-            bool visible = true;
-            if(ScimAction * action = dynamic_cast<ScimAction *>(exta[i])){
-                if(!action->visible() || !action->currentShown())
-                    visible = false;
-            }
-            if(visible)
-                m_insertedActions.append(exta[i]);
+    m_insertedActions.clear();
+    for(uint i=0; i<exta.size(); i++) {
+        bool visible = true;
+        if(ScimAction * action = dynamic_cast<ScimAction *>(exta[i])){
+            if(!action->visible() || !action->currentShown())
+                visible = false;
         }
-
-        if(m_insertedActions.count())
-            m_updatePropertiesNeeded = true;
-
-        unplugActionList(m_actionListName.latin1());
-
-        if(m_updatePropertiesNeeded && m_contentIsVisible)
-            show();
-    }
-    else
-    {
-        if(isFrontEndProperties)
-            m_updateFrontendProperties = true;
-        else
-            m_updateGUIProperties = true;
+        if(visible)
+            m_insertedActions.append(exta[i]);
     }
+
+    if(m_insertedActions.count())
+        m_updatePropertiesNeeded = true;
+
+    unplugActionList(m_actionListName);
+
+    if(m_updatePropertiesNeeded && m_contentIsVisible)
+        show();
+
+    //FIXME: see the FIXME in MainWindow::show()
+    if(isVisible())
+        QTimer::singleShot( 100, this, SLOT( adjustSize() ) );
 }
 
 void MainWindow::changeDirection(QBoxLayout::Direction d)
@@ -599,7 +577,7 @@ void MainWindow::appletDestroyed(bool appClosing)
         {
             //TODO: kicker is shutting down, may be caused by logout of KDE, crash of kicker or killed by users
             toggleDocking();
-            //FIXME: ugly hack: why the m_toolbar is so small after the previous call? Any why call adjustSize here
+            //FIXME: ugly hack: why the m_toolbar is so small after the previous call? And why call adjustSize here
             //won't do any good either: so we have to use a timer. This may be a potential bug
             QTimer::singleShot(1000, this, SLOT(adjustSize()));
         }
