@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2003-2005 by liuspider                                  *
+ *   Copyright (C) 2003-2006 by liuspider                                  *
  *   liuspider@users.sourceforge.net                                       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -479,10 +479,10 @@ void ScimIMEngineSettings::get_factory_list (QStringList &uuids,
 
     // Add "English/European" factory first.
     factory = new scim::ComposeKeyFactory ();
-    uuids.push_back (factory->get_uuid ());
+    uuids.push_back (QString::fromLatin1(factory->get_uuid ().c_str()));
     names.push_back (QString::fromUtf8(scim::utf8_wcstombs (factory->get_name ()).c_str()));
-    langs.push_back (scim::scim_get_normalized_language (factory->get_language ()));
-    icons.push_back (factory->get_icon_file ());
+    langs.push_back (QString::fromUtf8(scim::scim_get_normalized_language (factory->get_language ()).c_str()));
+    icons.push_back (QString::fromUtf8(factory->get_icon_file ().c_str()));
 
     for (size_t i = 0; i < module_list.size (); ++ i) {
         module.load (module_list [i], m_config);
@@ -502,7 +502,7 @@ void ScimIMEngineSettings::get_factory_list (QStringList &uuids,
                           QString::fromUtf8(scim::utf8_wcstombs (factory->get_name ()).c_str()));
                         langs.push_back (QString::fromUtf8(
                           scim::scim_get_normalized_language (factory->get_language ()).c_str()));
-                        icons.push_back (factory->get_icon_file ());
+                        icons.push_back (QString::fromUtf8(factory->get_icon_file ().c_str()));
                     }
                     factory.reset ();
                 }
