@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2003-2005 by liuspider                                  *
+ *   Copyright (C) 2003-2006 by liuspider                                  *
  *   liuspider@users.sourceforge.net                                       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -19,7 +19,8 @@
 #include <kapplication.h>
 
 SkimGlobalActions::SkimGlobalActions(QObject *parent, const char *name)
-: KActionCollection(0, parent, name), m_AboutApp(0), m_helpDialog(0) {
+: KActionCollection(0, parent, name), m_AboutApp(0), m_helpDialog(0),
+  updateFrontendPropertiesNeeded(false), updateGUIPropertiesNeeded(false){
     m_mc = static_cast<SkimPluginManager *>(parent);
     m_inputServer = m_mc->getInputServer();
 
@@ -70,6 +71,8 @@ SkimGlobalActions::SkimGlobalActions(QObject *parent, const char *name)
 
     //only after loadAllPlugins, overload standalone module list can be established
     connect(m_inputServer,SIGNAL(standaloneHelpersChanged()), this, SLOT(initStantaloneHelperActions()));
+
+    connect(m_inputServer, SIGNAL( transaction_end() ), SLOT( endTransaction() ));
 }
 
 void SkimGlobalActions::startSCIMConfigureHelper()
@@ -78,7 +81,7 @@ void SkimGlobalActions::startSCIMConfigureHelper()
 }
 
 void SkimGlobalActions::initStantaloneHelperActions() {
-    for(uint i=0; i<m_helperActions->count();i++)
+    for(uint i=0; i<m_helperActions->count();++i)
     {
         m_helperActions->action(i)->unplugAll();
     }
@@ -109,7 +112,6 @@ void SkimGlobalActions::initStantaloneHelperActions() {
 }
 
 SkimGlobalActions::~SkimGlobalActions() {
-    ScimKdeSettings::writeConfig();
     m_HelpMenu->deleteLater();
     if( m_helpDialog )
         m_helpDialog->deleteLater();
@@ -137,7 +139,7 @@ void SkimGlobalActions::receiveFactoryMenu(const std::vector <scim::PanelFactory
         m_serverAction->clear();
 
         if(groups.size() == 1)
-            for(uint i = 0; i < menu.size(); i++) {
+            for(uint i = 0; i < menu.size(); ++i) {
                 m_serverAction->insertItem("/" + QString::fromLatin1(menu[i].uuid.c_str()),
                                            KGlobal::iconLoader()->loadIcon(
                                                    QString::fromLocal8Bit(menu[i].icon.c_str()), KIcon::Small),
@@ -187,10 +189,12 @@ void SkimGlobalActions::propertyActivated(int id) {
 
 void SkimGlobalActions::deleteProperties(int client) {
     bool changed = false;
+    emit preparePropertiesRemove(client < 0);
+
     if(m_property_repository.contains(client)) {
         ScimAction * action;
         for(PropertyRepository::iterator it = m_property_repository[client].begin ();
-        it != m_property_repository[client].end (); it++) {
+        it != m_property_repository[client].end (); ++it) {
             switch (it->objectType) {
                 case scim_kde::Action:
                 case scim_kde::ComboAction:
@@ -200,6 +204,7 @@ void SkimGlobalActions::deleteProperties(int client) {
                     else
                         m_guiPropertyActions.remove(action);
                     changed = true;
+
                     action->unplugAll();
                     action->deleteLater();
                 break;
@@ -215,35 +220,19 @@ void SkimGlobalActions::deleteProperties(int client) {
 }
 
 void SkimGlobalActions::registerProperties(const scim::PropertyList & properties, int client) {
-    bool included = true;
-
-    scim::PropertyList::const_iterator pit = properties.begin ();
-
-    PropertyRepository::iterator prit;
-    if(m_property_repository.contains(client)) {
-        //find out existing properties and update them
-        for (; pit != properties.end (); ++pit) {
-            prit = std::find(m_property_repository[client].begin(), m_property_repository[client].end(),*pit);
-            if( prit != m_property_repository[client].end())
-            updateProperty (prit, *pit, client);
-            else
-            included = false;
-        }
-    } else {
-        included = false;
-    }
-
-    if (!included)  // Construct new properties.
-        create_properties (0, m_property_repository,
-                           properties, properties.begin(),
-                           properties.end(), client);
+    create_properties (m_property_repository,
+                          properties, properties.begin(),
+                          properties.end(), client);
 
     if(client < 0) { //registering a new set of frontend properties,
+        scim::PropertyList::const_iterator pit = properties.begin ();
+        PropertyRepository::iterator prit;
+
         m_frontendPropertyActions.clear();
         pit = properties.begin ();
 
         if(m_property_repository.contains(client)) {
-            //add all registered properties to frontendPropertyActions
+            //add all registered properties to m_frontendPropertyActions
             for (; pit != properties.end (); ++pit) {
                 prit = std::find(m_property_repository[client].begin(), m_property_repository[client].end(),*pit);
                 if( prit != m_property_repository[client].end())
@@ -266,7 +255,7 @@ void SkimGlobalActions::registerProperties(const scim::PropertyList & properties
     emit propertiesRegistered(client < 0);
 }
 
-void SkimGlobalActions::create_properties(QWidget * /*parent*/, ClientPropertyRepository &repository,
+void SkimGlobalActions::create_properties(ClientPropertyRepository &repository,
                                           const scim::PropertyList &/*properties*/, 
                                           scim::PropertyList::const_iterator  begin,
                                           scim::PropertyList::const_iterator  end,
@@ -294,6 +283,7 @@ void SkimGlobalActions::create_properties(QWidget * /*parent*/, ClientPropertyRe
                   if((eNodeIt = 
                         std::find(repository[client].begin(),  repository[client].end(),*it)) == repository[client].end()) {
                     info.globalUniqueID = root_property_id++;
+
                     node = new ScimComboAction(text, "", 0, 0, m_externalActions, it->get_key ().c_str());
                     if(client >= 0)
                         m_guiPropertyActions.push_back(node);
@@ -349,6 +339,8 @@ void SkimGlobalActions::create_properties(QWidget * /*parent*/, ClientPropertyRe
                     m_global_property_dict[info.globalUniqueID] = dictcand;
                     repository[client].push_back(info);
                     } else {  //existing property has been updated in the registerProperties function, so nothing to do here
+//                         std::cout << "existing property has been updated in the registerProperties function, so nothing to do here\n";
+//                         updateProperty (eNodeIt, *it, client);
                     }
                   }
                 } else {  //no child, create a std action
@@ -374,6 +366,7 @@ void SkimGlobalActions::create_properties(QWidget * /*parent*/, ClientPropertyRe
                     repository[client].push_back(info);
                       connect(node, SIGNAL(activated(int)), SLOT(propertyActivated(int)));
                   } else {  //existing property has been updated in the registerProperties function, so nothing to do here
+//                       updateProperty (eNodeIt, *it, client);
                   }
                 }
             it = next;
@@ -382,6 +375,21 @@ void SkimGlobalActions::create_properties(QWidget * /*parent*/, ClientPropertyRe
     }
 }
 
+void SkimGlobalActions::endTransaction()
+{
+    if(updateFrontendPropertiesNeeded)
+    {
+        updateFrontendPropertiesNeeded = false;
+        emit propertyChanged(true);
+    }
+
+    if(updateGUIPropertiesNeeded)
+    {
+        updateGUIPropertiesNeeded = false;
+        emit propertyChanged(false);
+    }
+}
+
 void SkimGlobalActions::updateProperty(PropertyRepository::iterator &it,
   const scim::Property &property, int client) {
     QString text = QString::fromUtf8(property.get_label ().c_str());
@@ -401,7 +409,10 @@ void SkimGlobalActions::updateProperty(PropertyRepository::iterator &it,
           if(action->visible() != property.visible() || action->currentShown() == false) {
             action->setCurrentShown(true);
             action->setVisible(property.visible());
-            emit propertyChanged(client < 0);
+            if(client < 0)
+                updateFrontendPropertiesNeeded = true;
+            else
+                updateGUIPropertiesNeeded = true;
           }
         break;
       case scim_kde::ComboAction:
@@ -415,7 +426,10 @@ void SkimGlobalActions::updateProperty(PropertyRepository::iterator &it,
           if(comboaction->visible() != property.visible() || comboaction->currentShown() == false) {
             comboaction->setCurrentShown(true);
             comboaction->setVisible(property.visible());
-            emit propertyChanged(client < 0);
+            if(client < 0)
+                updateFrontendPropertiesNeeded = true;
+            else
+                updateGUIPropertiesNeeded = true;
           }
         break;
       case scim_kde::Popup:
@@ -461,8 +475,8 @@ void SkimGlobalActions::SetFactoryInfo( const scim::PanelFactoryInfo & info ) {
         old_uuid = info.uuid;
 
         //all displayed frontend properties have to be hidden, for the new IMEngine may not have any properties
-        scim::PropertyList emptyProperty;
-        registerProperties(emptyProperty, -1);
+        m_frontendPropertyActions.clear();
+        emit propertiesRegistered(true); //update toolbar etc
     }
 }
 
