@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2003-2005 by liuspider                                  *
+ *   Copyright (C) 2003-2006 by liuspider                                  *
  *   liuspider@users.sourceforge.net                                       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -35,7 +35,49 @@
 using namespace scim;
 
 #include <qvaluestack.h>
-class SkimPluginManager::SkimPluginManagerPrivate
+
+#ifdef DEBUG_SESSIONMANAGEMENT
+#include <qfile.h>
+#endif
+
+class SkimSessionManaged : KSessionManaged
+{
+public:
+    SkimSessionManaged() : m_committed(false)
+    {
+#if DEBUG_SESSIONMANAGEMENT
+        QFile f("/tmp/commitData");
+        if ( f.open( IO_WriteOnly | IO_Append ) ) {
+            QTextStream stream( &f );
+            stream << "\nSkimSessionManaged started\n";
+            f.close();
+        }
+#endif
+    }
+    bool commitData(QSessionManager&)
+    {
+        m_committed = true;
+#if DEBUG_SESSIONMANAGEMENT
+        QFile f("/tmp/commitData");
+        if ( f.open( IO_WriteOnly | IO_Append ) ) {
+            QTextStream stream( &f );
+            stream << "commitData(QSessionManager&)\n";
+            f.close();
+        }
+#endif
+        ScimKdeSettings::writeConfig();
+        return true;
+    }
+
+    bool dataCommitted()
+    {
+        return m_committed;
+    }
+private:
+    bool m_committed;
+};
+
+class SkimPluginManagerPrivate
 {
 public:
   // All available plugins, regardless of category, and loaded or not
@@ -44,20 +86,10 @@ public:
   // Dict of all currently loaded plugins, mapping the SkimPluginInfo to
   // a plugin
   QMap<SkimPluginInfo *, SkimPlugin *> loadedPlugins;
-//   struct pluginActionInfo {
-//     KAction * action;
-//     QString slot;
-//     SkimPluginInfo * pinfo;
-//   };
-//   struct extraPluginInfo {
-//     bool hasActions;
-//     bool onDemand;
-//     QValueList<int> actionIDs;
-//   };
+
   QMap<int, QPair<SkimPluginInfo *, int> > pluginActionInfoRep;
   QMap<int, KAction * > pluginActionRepository;
-//   QMap<SkimPluginInfo *, extraPluginInfo> pluginActionInfo2ExtraInfo;
-//   int pluginActionInfoRep_id_counter;
+
   // The plugin manager's mode. The mode is StartingUp until loadAllPlugins()
   // has finished loading the plugins, after which it is set to Running.
   // ShuttingDown and DoneShutdown are used during skim shutdown by the
@@ -68,7 +100,7 @@ public:
   // Plugins pending for loading
   QValueStack<QString> pluginsToLoad;
   QValueList<int> pluginActionsToActivate;
-  
+
   // These two lists are used to overwrite enabled plugins set in the config file
   // and if one plugin keyname are included in both of them, it will be enabled
   QStringList forceEnabledPlugins;
@@ -124,7 +156,7 @@ SkimPluginManager* SkimPluginManager::self()
 {
     if ( !m_self )
       new SkimPluginManager();
-  
+
     return m_self;
 }
 
@@ -141,23 +173,23 @@ scim::ConfigPointer SkimPluginManager::scimConfigObject()
         }
     } else
         std::cerr << "  Failed to load Config Module 'kconfig'\n";
-    
+
     if (m_config.null ()) m_config = new scim::DummyConfig ();
-    
+
     return m_config;
 }
 
 SkimPluginManager::SkimPluginManager( QStringList includePlugins, QStringList excludePlugins,
    QStringList otherArgs, QObject *, const char *name)
- : QObject(0, name), aboutData(*(KGlobal::instance()->aboutData()))
+    : QObject(0, name), aboutData(*(KGlobal::instance()->aboutData())),
+      d(new SkimPluginManagerPrivate), m_sm(new SkimSessionManaged)
 {
     if ( !m_self )
       m_self = this;
-    d = new SkimPluginManagerPrivate;
+
     d->forceEnabledPlugins = includePlugins;
     d->forceDisabledPlugins = excludePlugins;
     d->shutdownMode = SkimPluginManagerPrivate::StartingUp;
-//     d->pluginActionInfoRep_id_counter = 0;
     d->forcedRun = false;
 
     //the correct display for this program should have been set properly before this point (set by qapplication)
@@ -244,8 +276,19 @@ SkimPluginManager::~SkimPluginManager()
       if(!InputServer->wait(3000))
         std::cerr << "Input Socket Server exited abnormally!\n";
 
-    ScimKdeSettings::writeConfig();
+#if DEBUG_SESSIONMANAGEMENT
+    QFile f("/tmp/commitData");
+    if ( f.open( IO_WriteOnly | IO_Append ) ) {
+        QTextStream stream( &f );
+        stream << "SkimPluginManager::~SkimPluginManager() " << m_sm->dataCommitted() << "\n stoped \n";
+        f.close();
+    }
+#endif
+    //check whether the configure is already saved when logout the session
+    if(!m_sm->dataCommitted())
+        ScimKdeSettings::writeConfig();
 
+    delete m_sm;
     delete ScimKdeSettings::self();
     kdDebug( 18018 ) << k_lineinfo << "SkimPluginManager exited!" << endl;
     kapp->quit();
@@ -255,7 +298,7 @@ SkimPluginInfo::List SkimPluginManager::availablePlugins( const QString &categor
 {
     if ( category.isEmpty() )
       return d->plugins;
-  
+
     SkimPluginInfo::List result;
     SkimPluginInfo::List::ConstIterator it;
     for ( it = d->plugins.begin(); it != d->plugins.end(); ++it )
@@ -263,7 +306,7 @@ SkimPluginInfo::List SkimPluginManager::availablePlugins( const QString &categor
       if ( ( *it )->category() == category )
         result.append( *it );
     }
-  
+
     return result;
 }
 
@@ -276,7 +319,7 @@ QMap<SkimPluginInfo *, SkimPlugin *> SkimPluginManager::loadedPlugins( const QSt
       if ( category.isEmpty() || it.key()->category() == category )
         result.insert( it.key(), it.data() );
     }
-  
+
     return result;
 }
 
@@ -304,7 +347,7 @@ void SkimPluginManager::shutdown()
       it.data()->aboutToUnload();
       it = nextIt;
     }
-  
+
     QTimer::singleShot( 3000, this, SLOT( slotShutdownTimeout() ) );
 }
 
@@ -319,7 +362,7 @@ void SkimPluginManager::slotPluginReadyForUnload()
       kdWarning( 18018 ) << k_lineinfo << "Calling object is not a plugin!" << endl;
       return;
     }
-  
+
     plugin->deleteLater();
 }
 
@@ -329,15 +372,15 @@ void SkimPluginManager::slotShutdownTimeout()
     // Do nothing in that case.
     if ( d->shutdownMode == SkimPluginManagerPrivate::DoneShutdown )
       return;
-  
+
     QStringList remaining;
     for ( QMap<SkimPluginInfo *, SkimPlugin *>::ConstIterator it = d->loadedPlugins.begin(); it != d->loadedPlugins.end(); ++it )
       remaining.append( it.data()->pluginId() );
-  
+
     kdWarning( 18018 ) << k_lineinfo << "Some plugins didn't shutdown in time!" << endl
       << "Remaining plugins: " << remaining.join( QString::fromLatin1( ", " ) ) << endl
       << "Forcing skim shutdown now." << endl;
-  
+
     slotShutdownDone();
 }
 
@@ -354,7 +397,7 @@ void SkimPluginManager::slotShutdownDone()
 void SkimPluginManager::reloadAllPlugins() {
 
     d->shutdownMode = SkimPluginManagerPrivate::Reloading;
-    
+
     // Ask all plugins to unload
     QMap<SkimPluginInfo *, SkimPlugin *>::ConstIterator it;
     for ( it = d->loadedPlugins.begin(); it != d->loadedPlugins.end(); /* EMPTY */ )
@@ -382,7 +425,6 @@ void SkimPluginManager::loadAllPlugins()  // FIXME: We need session management h
     {
         QString key = ( *it )->pluginName();
         ( *it )->load(config);
-//         QVariant v;
 
         if ( d->forceDisabledPlugins.contains(key) && ( *it )->isNoDisplay())
         {
@@ -401,9 +443,7 @@ void SkimPluginManager::loadAllPlugins()  // FIXME: We need session management h
 
                     for(uint ita = 0; ita < actions.size(); ita++) {
                         SkimPluginActionInfo & actinfo = actions[ita];
-//                         kdesk.setActionGroup(actinfo);
-//                         int id = actinfo.id;
-//                         einfo.actionIDs.push_back(id);
+
                         QIconSet icon(kil->loadIcon(actinfo.icon, KIcon::Toolbar));
                         if(( *it )->isOnDemand()) {
                             ScimAction * action = new ScimAction(actinfo.name, icon,
@@ -424,7 +464,6 @@ void SkimPluginManager::loadAllPlugins()  // FIXME: We need session management h
                             }
                         }
                         d->pluginActionInfoRep[actinfo.id] = QPair<SkimPluginInfo*, int> (*it, ita);
-                        
                     }
 //                     d->pluginActionInfo2ExtraInfo[*it] = einfo;
                 }
@@ -456,10 +495,10 @@ void SkimPluginManager::slotLoadNextPlugin()
       }
       return;
     }
-  
+
     QString key = d->pluginsToLoad.pop();
     loadPluginInternal( key );
-  
+
     // Schedule the next run unconditionally to avoid code duplication on the
     // allPluginsLoaded() signal's handling. This has the added benefit that
     // the signal is delayed one event loop, so the accounts are more likely
@@ -470,7 +509,7 @@ void SkimPluginManager::slotLoadNextPlugin()
 SkimPlugin * SkimPluginManager::loadPlugin( const QString &_pluginId, PluginLoadMode mode /* = LoadSync */ )
 {
     QString pluginId = _pluginId;
-  
+
     if ( mode == LoadSync )
     {
       return loadPluginInternal( pluginId );
@@ -515,14 +554,7 @@ SkimPlugin *SkimPluginManager::loadPluginInternal( const QString &pluginId )
           SkimPluginActionInfo & ainfo = info->actions()[i];
           connect(d->pluginActionRepository[ainfo.id], SIGNAL(activated()), plugin, ainfo.slot.latin1());
       }
-//       /*if(d->pluginActionInfo2ExtraInfo.contains(info)) */{
-//         for(uint i = 0; i < d->pluginActionInfo2ExtraInfo[info].actionIDs.size(); i++) {
-//           int id = d->pluginActionInfo2ExtraInfo[info].actionIDs[i];
-//           SkimPluginManagerPrivate::pluginActionInfo & painfo = d->pluginActionInfoRep[id];
-//           connect(painfo.action, SIGNAL(activated()), plugin, painfo.slot.latin1());
-//         }
-//       }
-      
+
       for(uint i = 0; i< d->pluginActionsToActivate.size(); ++i) {
         int id = d->pluginActionsToActivate[i];
         if(info == d->pluginActionInfoRep[id].first) {
@@ -642,7 +674,7 @@ QString SkimPluginManager::pluginName( const SkimPlugin *plugin ) const
       if ( it.data() == plugin )
         return it.key()->name();
     }
-  
+
     return QString::fromLatin1( "Unknown" );
 }
 
@@ -654,7 +686,7 @@ QString SkimPluginManager::pluginId( const SkimPlugin *plugin ) const
       if ( it.data() == plugin )
         return it.key()->pluginName();
     }
-  
+
     return QString::fromLatin1( "unknown" );
 }
 
@@ -666,7 +698,7 @@ QString SkimPluginManager::pluginIcon( const SkimPlugin *plugin ) const
       if ( it.data() == plugin )
         return it.key()->icon();
     }
-  
+
     return QString::fromLatin1( "Unknown" );
 }
 
@@ -678,23 +710,23 @@ SkimPluginInfo * SkimPluginManager::infoForPluginId( const QString &pluginId ) c
       if ( ( *it )->pluginName() == pluginId )
         return *it;
     }
-  
+
     return 0L;
 }
 
 bool SkimPluginManager::setPluginEnabled( const QString &_pluginId, bool enabled /* = true */ )
 {
     QString pluginId = _pluginId;
-  
+
     KConfig *config = KGlobal::config();
     config->setGroup( "Plugins" );
-  
+
     if ( !infoForPluginId( pluginId ) )
       return false;
-  
+
     config->writeEntry( pluginId/* + QString::fromLatin1( "Enabled" )*/, enabled );
     config->sync();
-  
+
     return true;
 }
 
